// $Id: Application.cpp,v 1.60 2014/02/10 10:34:30 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "Application.moc"

#include "ConfigurationDialog.h"
#include "Config.h"
#include "Debug.h"
#include "File.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "XmlOptions.h"
#include "InformationDialog.h"
#include "QuestionDialog.h"
#include "TaskBarBase.h"
#include "TaskBarConfiguration.h"
#include "TaskBarDialog.h"
#include "TaskBarFixed.h"
#include "TaskBarMinimum.h"
#include "TaskBarParabolic.h"
#include "TaskParser.h"

#include <QApplication>
#include <QMessageBox>

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( 0, arguments ),
    Counter( "Application" )
{
    Debug::Throw( "Application::Application.\n" );
    connect( qApp, SIGNAL(aboutToQuit()), this, SLOT(_aboutToQuit()) );
    qApp->setQuitOnLastWindowClosed( false );
}

//____________________________________________
Application::~Application( void )
{
    Debug::Throw( "Application::~Application.\n" );
}

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    /* try load the file from command line, and default file if it fails */
    QStringList filenames( commandLineParser( _arguments() ).orphans() );
    if( !filenames.isEmpty() ) XmlOptions::get().setRaw( "XML_FILE", filenames.front() );

    // change close action shortcut
    closeAction().setShortcut( QKeySequence() );

    addTaskBarAction_ = new QAction( IconEngine::get( IconNames::Add ), tr( "Add Taskbar" ), this );
    connect( addTaskBarAction_, SIGNAL(triggered()), SLOT(_addTaskBar()) );

    removeTaskBarAction_ = new QAction( IconEngine::get( IconNames::Remove ), tr( "Remove Taskbar" ), this );
    connect( removeTaskBarAction_, SIGNAL(triggered()), SLOT(_removeTaskBar()) );

    // load taskBars
    _readTaskBars();

    // configuration
    emit configurationChanged();

    // show all taskBars
    foreach( TaskBarBase* taskBar, taskBars_ )
    { taskBar->show(); }

    return true;

}

//____________________________________________
void Application::usage( void ) const
{
    _usage( "taskbar", tr( "[options] [configuration file]" ) );
    commandLineParser().usage();
    return;
}

//___________________________________________________
void Application::_configuration( void )
{
    Debug::Throw( "Application::_configuration.\n" );

    emit saveConfiguration();

    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL(configurationChanged()), this, SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//_______________________________________________
void Application::_addTaskBar( void )
{

    Debug::Throw( "Application::_addTaskBar.\n" );
    TaskBarDialog dialog( currentTaskBar() );
    if( !dialog.exec() ) return;

    // create new taskBar, put in list and write to file
    TaskBarBase *bar = _newTaskBar( dialog.taskBarConfiguration().attributes() );
    taskBars_ << bar;
    bar->updateConfiguration();
    bar->show();
    _writeTaskBars();

    // update actions
    _updateActions();

}

//_______________________________________________
void Application::_editTaskBar( const TaskBarAttributes& attributes )
{

    Debug::Throw( "Application::_editTaskBar.\n" );

    // retrieve current TaskBar
    TaskBarBase* bar( currentTaskBar() );
    Debug::Throw( "Application::_editTaskBar - got bar.\n" );

    // check if attributes are changed
    if( attributes == bar->attributes() )
    {
        Debug::Throw( "Application::_editTaskBar - attributes are unchanged.\n" );
        return;
    }

    // update current task bar (the TaskAttributesList is unchanged)
    TaskAttributes::List task_list( bar->attributes().taskAttributesList() );
    _setCurrentAttributes( attributes );
    _currentAttributes().setTaskAttributesList( task_list );

    Debug::Throw( "Application::_editTaskBar - attributes set.\n" );

    // if taskBar mode differ, delete taskBar, create a new one and replace
    // otherwise, just update attributes
    if( _currentAttributes().mode() != bar->attributes().mode() )
    {

        _removeTaskBar( false );
        bar = _newTaskBar( _currentAttributes() );
        bar->updateConfiguration();
        taskBars_ << bar;
        bar->show();

    } else { bar->setAttributes( _currentAttributes() ); }

    // save taskBars
    _writeTaskBars();

    Debug::Throw( "Application::_editTaskBar - done.\n" );
    return;

}

//_______________________________________________
void Application::_removeTaskBar( const bool& ask )
{
    Debug::Throw( "Application::_removeTaskBar.\n" );

    // retrieve current TaskBar
    TaskBarBase* bar( currentTaskBar() );

    if( ask )
    {
        const QString buffer = QString( tr( "Remove taskBar '%1' ?" ) ).arg(  _currentAttributes().name() );
        if( !QuestionDialog( bar, buffer ).exec() ) return;
    }

    // delete taskBars matching attributes
    TaskBarBase::List tmp;
    foreach( TaskBarBase* taskBar, taskBars_ )
    {
        if( taskBar->attributes().id() == _currentAttributes().id() ) taskBar->deleteLater();
        else tmp << taskBar;
    }

    // replace
    taskBars_ = tmp;

    // write new set of taskBars to file
    _writeTaskBars();

    // update actions
    _updateActions();

}

//_______________________________________________
void Application::_readTaskBars( void )
{

    File filename( XmlOptions::get().raw("XML_FILE" ) );
    Debug::Throw() << "Application::_readTaskBars - file: " << filename << endl;

    // make sure file is hidden (windows only)
    if( filename.localName().startsWith( '.' ) ) filename.setHidden();

    // read tasks from db file
    TaskParser parser;
    parser.read( filename );

    // check errors
    if( parser.xmlError() )
    {
        const QString buffer = QString( tr( "Errors occured while parsing file:\n%1" ) ).arg( parser.xmlError().toString() );
        InformationDialog( 0, buffer ).exec();
    }

    // local copy
    const TaskBarAttributes::List& attributes( parser.attributes() );
    TaskBarAttributes::List::const_iterator iter = attributes.begin();
    TaskBarBase::List::iterator barIter = taskBars_.begin();

    // update existing taskBars
    while( iter != attributes.end() && barIter != taskBars_.end() )
    {

        // check if zoom mode match
        if( (*barIter)->attributes().mode() != iter->mode() )
        {

            (*barIter)->deleteLater();
            *barIter = _newTaskBar( *iter );

        } else {

            (*barIter)->setAttributes( *iter );

        }

        (*barIter)->show();
        ++iter;
        ++barIter;

    }

    if( iter != attributes.end() )
    {

        // additional taskBar if needed
        for( ; iter != attributes.end(); ++iter )
        {
            TaskBarBase *bar = _newTaskBar( *iter );
            taskBars_ << bar;
        }

    } else if( barIter != taskBars_.end() ) {

        // remove excedent taskBar if needed
        for( TaskBarBase::List::iterator barIter2 = barIter; barIter2 != taskBars_.end(); barIter2++ )
        { (*barIter2)->deleteLater(); }

        taskBars_.erase( barIter, taskBars_.end() );

    }

    // update actions
    _updateActions();

    return;

}

//_______________________________________________
void Application::_writeTaskBars( bool check ) const
{

    QString filename( XmlOptions::get().raw("XML_FILE" ) );
    Debug::Throw() << "Application::_writeTaskBars - filename: " << filename << endl;

    // if taskBars have not been initialized, do nothing
    if( !_realized() )
    {
        Debug::Throw( "Application::_writeTaskBars - application not realized. Aborted.\n" );
        return;
    }

    // check taskBar
    if( taskBars_.empty() )
    {
        Debug::Throw( "Application::_writeTaskBars - taskBar list empty.\n" );
        return;
    }

    // try open XML file
    File file( filename );

    TaskParser parser;
    foreach( TaskBarBase* taskBar, taskBars_ )
    {
        taskBar->synchronizeAttributes();
        parser.addAttributes( taskBar->attributes() );
    }

    bool written( parser.write( file ) );
    if( (!written) && check )
    {
        const QString buffer = QString( tr( "Unable to write to file '%1'." ) ).arg( file );
        InformationDialog( 0, buffer ).exec();
    }

    return;

}

//_______________________________________________
void Application::_aboutToQuit( void )
{
    Debug::Throw( "Application::_AboutToQuit.\n" );
    _writeTaskBars();
}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    { return true; }

    return false;

}

//_______________________________________________
TaskBarBase* Application::_newTaskBar( const TaskBarAttributes& attributes )
{

    Debug::Throw( "Application::_newTaskBar.\n" );

    TaskBarBase* bar = 0;
    switch ( attributes.mode() )
    {

        default:
        case TaskBarAttributes::Fixed:
        bar = new TaskBarFixed();
        break;


        case TaskBarAttributes::Minimum:
        bar = new TaskBarMinimum();
        break;

        case TaskBarAttributes::Parabolic:
        bar = new TaskBarParabolic();
        break;

    }

    connect( bar, SIGNAL(modified()), this, SLOT(_writeTaskBars()) );
    connect( bar, SIGNAL(taskBarEntered(TaskBarAttributes)), SLOT(_setCurrentAttributes(TaskBarAttributes)) );
    connect( bar, SIGNAL(attributesChanged(TaskBarAttributes)), SLOT(_editTaskBar(TaskBarAttributes)) );
    connect( bar, SIGNAL(taskListChanged()), SLOT(_writeTaskBars()) );

    bar->setWindowIcon( applicationIcon() );
    bar->setWindowTitle( applicationName() );
    bar->setAttributes( attributes, false );

    // close action is added to the bar to enable the shortcut
    bar->addAction( &closeAction() );

    return bar;

}

//_______________________________________________
TaskBarBase* Application::_taskBar( const TaskBarAttributes& attributes ) const
{
    Debug::Throw( "Application::_taskBar.\n" );
    TaskBarBase::List::const_iterator iter( std::find_if( taskBars_.begin(), taskBars_.end(), TaskBarBase::SameAttributesFTor( attributes ) ) );
    Q_ASSERT( iter != taskBars_.end() );

    Debug::Throw( "Application::_taskBar - found.\n" );
    return *iter;

}


//_______________________________________________
void Application::_updateActions( void ) const
{
    Debug::Throw( "Application::_updateAction.\n" );
    removeTaskBarAction().setEnabled( taskBars_.size() > 1 );
}
