// $Id: ConfigurationDialog.cpp,v 1.51 2015/01/05 10:09:41 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConfigurationDialog.h"
#include "ConfigurationDialog.moc"

#include "Debug.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "OptionCheckBox.h"
#include "OptionRadioButton.h"
#include "OptionSlider.h"
#include "OptionSpinBox.h"
#include "SvgConfiguration.h"
#include "TransparencyConfiguration.h"

#include <QButtonGroup>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QPushButton>

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent ):
    BaseConfigurationDialog( parent )
{

    Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );
    setWindowTitle( tr( "Configuration - Taskbar" ) );

    // base configuration
    baseConfiguration( 0 );

    // transparency configuration
    QWidget* page;
    Transparency::TransparencyConfiguration* transparency_configuration;

    page = &addPage( IconEngine::get( IconNames::PreferencesColors ), tr( "Foreground" ), tr( "Foreground color settings" ) );
    page->layout()->addWidget( transparency_configuration = new Transparency::TransparencyConfiguration( page ));
    addOptionWidget( transparency_configuration );

    // generic widgets
    QWidget *box;
    GridLayout* gridLayout;
    OptionSpinBox* spinbox;
    OptionCheckBox* checkBox;
    OptionSlider* slider;

    // svg background
    page = &addPage( IconEngine::get( IconNames::PreferencesBackground ), tr( "Background" ), tr( "Window background settings" ) );
    Svg::SvgConfiguration* svgConfiguration( new Svg::SvgConfiguration( page ) );
    page->layout()->addWidget( svgConfiguration);
    addOptionWidget( svgConfiguration );

    // buttons
    page = &addPage( IconEngine::get( IconNames::PreferencesAppearance ), tr( "Appearance" ), tr( "Window appearance" ) );
    box = new QGroupBox( tr( "Button Size" ), page );
    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
    box->setLayout( gridLayout );
    page->layout()->addWidget( box );

    QLabel* label;
    gridLayout->addWidget( label = new QLabel( tr( "Margin:" ), box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "MARGIN" ) );
    spinbox->setSuffix( tr( "px" ) );
    spinbox->setToolTip( tr( "Space between buttons and window border" ) );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    gridLayout->addWidget( label = new QLabel( tr( "Spacing:" ), box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "SPACING" ) );
    spinbox->setSuffix( tr( "px" ) );
    spinbox->setToolTip( tr( "Space between consecutive buttons" ) );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    gridLayout->addWidget( label = new QLabel( tr( "Minimum size:" ), box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "MIN_BUTTON_SIZE" ) );
    spinbox->setToolTip( tr( "Button minimum size (pixels)" ) );
    spinbox->setSuffix( tr( "px" ) );
    spinbox->setMinimum(0);
    spinbox->setMaximum(128);
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    gridLayout->addWidget( new QLabel( tr( "Maximum size:" ), box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "MAX_BUTTON_SIZE" ) );
    spinbox->setToolTip( tr( "Button maximum size (pixels)" ) );
    spinbox->setSuffix( tr( "px" ) );
    spinbox->setMinimum(0);
    spinbox->setMaximum(256);
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    // button highlight
    box = new QGroupBox( tr( "Highlight" ), page );
    box->setLayout( new QVBoxLayout() );
    box->layout()->setMargin(5);
    box->layout()->setSpacing(5);
    page->layout()->addWidget( box );

    QHBoxLayout *hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    box->layout()->addItem( hLayout );

    QButtonGroup* button_group( new QButtonGroup( this ) );
    button_group->setExclusive( true );

    OptionRadioButton* radioButton;
    hLayout->addWidget( radioButton = new OptionRadioButton( tr( "None" ), box, "NO_HIGHLIGHT" ) );
    radioButton->setToolTip( tr( "Do not highlight buttons" ) );
    addOptionWidget( radioButton );
    button_group->addButton( radioButton );

    hLayout->addWidget( radioButton = new OptionRadioButton( tr( "On enter" ), box, "HIGHLIGHT_ON_ENTER" ) );
    radioButton->setToolTip( tr( "Highlight buttons when hovered" ) );
    addOptionWidget( radioButton );
    button_group->addButton( radioButton );

    hLayout->addWidget( radioButton = new OptionRadioButton( tr( "On click" ), box, "HIGHLIGHT_ON_CLICK" ) );
    radioButton->setToolTip( tr( "Highlight buttons when clicked" ) );
    addOptionWidget( radioButton );
    button_group->addButton( radioButton );

    hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    box->layout()->addItem( hLayout );

    hLayout->addWidget( label = new QLabel( tr( "Intensity:" ), box ) );
    hLayout->addWidget( slider = new OptionSlider( box, "HIGHLIGHT_INTENSITY" ) );
    slider->setScale( 100 );
    slider->setRange( 0, 100 );
    slider->setToolTip( tr( "Button highlight intensity" ) );
    addOptionWidget( slider );
    label->setBuddy( slider );

    // tooltips
    page = &addPage( IconEngine::get( IconNames::PreferencesUnsorted ), tr( "Unsorted" ), tr( "Additional unsorted settings" ) );
    box = new QWidget( page );
    box->setLayout( new QVBoxLayout() );
    box->layout()->setMargin(5);
    box->layout()->setSpacing(5);
    page->layout()->addWidget( box );

    box->layout()->addWidget( checkBox = new OptionCheckBox( tr( "Show tooltips" ), box, "SHOW_TOOLTIPS" ) );
    checkBox->setToolTip( tr( "Show tooltips" ) );
    addOptionWidget( checkBox );

    // misc
    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    box->layout()->addItem( gridLayout );

    gridLayout->addWidget( label = new QLabel( tr( "Position grid size:" ), box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "GRID" ) );
    spinbox->setSuffix( tr( "px" ) );
    spinbox->setMinimum(1);
    spinbox->setToolTip( tr( "Step size when moving the taskBar on desktop" ) );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    // load initial configuration
    read();

}
