
// $Id: TaskAttributes.cpp,v 1.40 2015/01/09 15:40:28 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Command.h"
#include "CustomPixmap.h"
#include "File.h"
#include "IconNames.h"
#include "TaskAttributes.h"
#include "XmlDef.h"
#include "XmlOptions.h"
#include "XmlTimeStamp.h"
#include "XmlString.h"

#include <QPixmap>

//___________________________________________________
unsigned int& TaskAttributes::_idCount( void )
{
    static unsigned int counter(0);
    return counter;
}

//___________________________________________
TaskAttributes::TaskAttributes( void ):
    Counter( "TaskAttributes" ),
    id_( _idCount()++ ),
    flags_( None )
{
    Debug::Throw( "TaskAttributes::TaskAttributes\n" );
    setCreation();
    setIconName();
}

//___________________________________________
TaskAttributes::TaskAttributes( const QDomElement& element ):
    Counter( "TaskAttributes" ),
    id_( _idCount()++ ),
    flags_( None )
{
    Debug::Throw( "TaskAttributes::TaskAttributes\n" );

    QString arguments;

    // parse attributes
    QDomNamedNodeMap attributes( element.attributes() );
    for( int i=0; i<attributes.count(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;
        QString name( attribute.name() );
        QString value( attribute.value() );

        if( name == Xml::Name ) setName( XmlString( value ).toText() );
        else if( name == Xml::Command ) setCommand( XmlString( value ).toText() );
        else if( name == Xml::Arguments ) arguments = XmlString( value ).toText();
        else if( name == Xml::Icon ) setIconName( value );
        else if( name == Xml::Flags ) setFlags( Flags(value.toInt()) );
        else Debug::Throw(0) << "TaskAttributes::TaskAttributes - unrecognized name " << name << endl;
    }

    // parse children elements
    for(QDomNode child_node = element.firstChild(); !child_node.isNull(); child_node = child_node.nextSibling() )
    {
        QDomElement child_element = child_node.toElement();
        QString tag_name( child_element.tagName() );
        if( tag_name == Xml::Comments ) setComments( XmlString( child_element.text() ).toText() );
        else if( tag_name == Xml::Modification ) setModification( XmlTimeStamp( child_element ) );
        else if( tag_name == Xml::Creation ) setCreation( XmlTimeStamp( child_element ) );
        else if( tag_name == Xml::LastCall ) setLastCalled( XmlTimeStamp( child_element ) );
        else Debug::Throw(0) << "TaskAttributes::TaskAttributes - unrecognized child " << child_element.tagName() << ".\n";
    }

    // add optional arguments to command
    // this is necessary to handle old arguments format
    if( !arguments.isEmpty() ) setCommand( command() + " " + arguments );

}

//___________________________________________
QDomElement TaskAttributes::domElement( QDomDocument& parent ) const
{

    Debug::Throw( "TaskAttributes::DomElement.\n");
    QDomElement out( parent.createElement( Xml::Task ) );
    out.setAttribute( Xml::Flags, QString::number( flags() ) );
    if( hasFlag( IsSeparator ) ) return out;

    // attributes
    if( !name().isEmpty() ) out.setAttribute( Xml::Name, XmlString( name() ).toXml() );
    if( !command().isEmpty() ) out.setAttribute( Xml::Command, XmlString( command() ).toXml() );
    if( !iconName().isEmpty() ) out.setAttribute( Xml::Icon, iconName() );

    // comments
    if( !comments().isEmpty() )
    {
        QDomElement comments_element = parent.createElement( Xml::Comments );
        QDomText comments_text = parent.createTextNode( XmlString( comments() ).toXml() );
        comments_element.appendChild( comments_text );
        out.appendChild( comments_element );
    }

    // timestamps
    if( creation().isValid() )
    {

        Debug::Throw() << "TaskAttributes::DomElement - name: " << name() << " creation: " << creation().toString() << " " << creation() << endl;
        out.appendChild( XmlTimeStamp( creation() ).domElement( Xml::Creation, parent ) );
    }

    if( modification().isValid() )
    {
        Debug::Throw() << "TaskAttributes::DomElement - name: " << name() << " modification: " << modification().toString() << " " << modification() << endl;
        out.appendChild( XmlTimeStamp( modification() ).domElement( Xml::Modification, parent ) );
    }

    if( lastCalled().isValid() )
    {
        Debug::Throw() << "TaskAttributes::DomElement - name: " << name() << " lastCalled: " << lastCalled().toString() << " " << lastCalled() << endl;
        out.appendChild( XmlTimeStamp( lastCalled() ).domElement( Xml::LastCall, parent ) );
    }
    return out;

}

//___________________________________________
bool TaskAttributes::differs( const TaskAttributes& other ) const
{

    if( name() != other.name() ) return true;
    else if( comments() != other.comments() ) return true;
    else if( command() != other.command() ) return true;
    else if( iconName() != other.iconName() ) return true;
    else if( flags() != other.flags() ) return true;
    else return false;

}

//___________________________________________
QString TaskAttributes::name( void ) const
{
    QString name( name_ );
    if( !name.isEmpty() ) name[0] = name[0].toUpper();
    return name;
}

//___________________________________________
QString TaskAttributes::comments( void ) const
{
    QString comments( comments_ );
    if( !comments.isEmpty() ) comments[0] = comments[0].toUpper();
    return comments;
}

//___________________________________________
void TaskAttributes::setIconName( const QString& value )
{

    if( CustomPixmap( value ).isNull() ) iconFilename_ = defaultIconFilename();
    else iconFilename_ = value;
    return;

}

//___________________________________________
QString TaskAttributes::defaultIconFilename( void ) const
{
    Debug::Throw( "TaskAttributes::defaultIconFilename.\n" );

    // set to (truncated) default name
    File file( IconNames::Default );
    QString out( IconNames::Default );

    // load icon path list from options
    Options::List pathList( XmlOptions::get().specialOptions( "PIXMAP_PATH" ) );
    if( pathList.empty() ) return out;

    // loop over path and find matching pixmaps
    foreach( const Option& option, pathList )
    {

        // skip empty path
        if( option.raw().isEmpty() ) continue;

        // prepare filename
        File iconFile;

        // see if path is internal resource path
        if( option.raw()[0] == ':' ) iconFile = file.addPath( File( option.raw() ) );
        else iconFile = File( option.raw() ).find( file, true );

        // load pixmap
        if( !( iconFile.isEmpty() && QPixmap( iconFile ).isNull() ) )
        {
            out = iconFile;
            break;
        }
    }

    Debug::Throw() << "TaskAttributes::defaultIconFilename - filename: " << out << endl;
    return out;

}

//__________________________________________________
bool TaskAttributes::exec( void )
{

    Debug::Throw( "TaskAttributes::exec.\n" );

    // check if separator
    if( hasFlag( IsSeparator ) ) return true;

    setLastCalled();
    if( command().isEmpty() ) return false;
    else return Command( command() ).run();
}

//___________________________________________
bool TaskAttributes::List::differs( const QList<TaskAttributes>& other ) const
{
    if( size() != other.size() ) return true;
    else {

        for( int index = 0; index < size(); ++index )
        { if( at(index).differs( other.at(index) ) ) return true; }

        return false;
    }

}
