#ifndef TaskAttributes_h
#define TaskAttributes_h

// $Id: TaskAttributes.h,v 1.30 2015/01/09 15:40:28 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Options.h"
#include "TimeStamp.h"

#include <QDomElement>
#include <QDomDocument>
#include <QList>

//* single TaskAttributes attributesrmation
class TaskAttributes: public Counter
{

    public:

    //* task attributes list
    class List: public QList<TaskAttributes>, public Counter
    {

        public:

        //* constructor
        List( void ):
            Counter( "TaskAttributes::List" )
        {}

        //* copy constructor
        List( const QList<TaskAttributes>& other ):
            QList<TaskAttributes>( other ),
            Counter( "TaskAttributes::List" )
            {}

        //* destructor
        virtual ~List( void )
        {}

        //* returns true if at least one item in lists ::differs from the other
        bool differs( const QList<TaskAttributes>& ) const;

    };

    //* task attributes list
    using ListIterator = QListIterator<TaskAttributes>;

    //* constructor
    TaskAttributes( void );

    //* constructor
    TaskAttributes( const QDomElement& element );

    //* dom element
    QDomElement domElement( QDomDocument& parent ) const;

    //* id
    const unsigned int& id() const
    { return id_; }

    //* equal to operator
    bool operator == (const TaskAttributes& attributes ) const
    {
        return
            id() == attributes.id()  &&
            name() == attributes.name() &&
            comments() == attributes.comments() &&
            command() == attributes.command() &&
            iconName() == attributes.iconName() &&
            flags() == attributes.flags();
    }

    //* less than operator
    bool operator < (const TaskAttributes& attributes ) const
    {
        if( id() != attributes.id() ) return id() < attributes.id();
        else if( name() != attributes.name() ) return name() < attributes.name();
        else if( comments() != attributes.comments() ) return comments() < attributes.comments();
        else if( command() != attributes.command() ) return command() < attributes.command();
        else if( iconName() != attributes.iconName() ) return iconName() < attributes.iconName();
        else if( flags() != attributes.flags() ) return flags() < attributes.flags();
        else return false;
    }

    //*@name accessors
    //@{

    //* return true if task attributes are different
    /*! unlike the == operator, it does not check on the ID */
    bool differs( const TaskAttributes& ) const;

    //* name
    QString name( void ) const;

    //* comments
    QString comments( void ) const;

    //* command
    const QString& command( void ) const
    { return command_; }

    //* load default icon filename
    QString defaultIconFilename( void ) const;

    //* icon
    const QString& iconName( void ) const
    { return iconFilename_; }

    //* creation
    const TimeStamp& creation( void ) const
    { return creation_; }

    //* modification
    const TimeStamp& modification( void ) const
    { return modification_; }

    //* last call
    const TimeStamp& lastCalled( void ) const
    { return lastCalled_; }

    //* status in list
    enum Flag
    {
        None = 0,
        ShowName = 1<<0,
        IsSeparator = 1<<1
    };

    Q_DECLARE_FLAGS( Flags, Flag );

    //* flags
    bool hasFlag( Flag flag ) const
    { return flags_ & flag; }

    //* flags
    Flags flags( void ) const
    { return flags_; }

    //@}

    //*@name modifiers
    //@{

    //* name
    void setName( const QString& value )
    { name_ = value; }

    //* comments
    void setComments( const QString& buf )
    { comments_ = buf ;}

    //* command
    void setCommand( const QString& value )
    { command_ = value; }

    //* icon
    void setIconName( const QString& value = QString() );

    //* creation
    void setCreation( const TimeStamp& stamp = TimeStamp::now() )
    { creation_ = stamp; }

    //* modification
    void setModification( const TimeStamp& stamp = TimeStamp::now() )
    { modification_ = stamp; }

    //* last call
    void setLastCalled( const TimeStamp& stamp = TimeStamp::now() )
    { lastCalled_ = stamp; }

    //* execute command
    bool exec( void );

    //* flags
    TaskAttributes& setFlag( const Flag& flag, const bool& value )
    {
        if( value ) flags_ |= flag;
        else flags_ &= (~flag);
        return *this;
    }

    //* flags
    void setFlags( Flags flags )
    { flags_ = flags; }

    //* used to count separators
    class IsSeparatorFTor
    {

        public:

        //* predicate
        bool operator() (const TaskAttributes& attributes ) const
        { return attributes.hasFlag( TaskAttributes::IsSeparator ); }

    };

    //@}

    private:

    //* id count
    static unsigned int& _idCount( void );

    //* id
    unsigned int id_;

    //* application name
    QString name_;

    //* comments
    QString comments_;

    //* application command
    QString command_;

    //* icon filename
    QString iconFilename_;

    //* flags
    Flags flags_;

    //*@name time stamps
    //@{

    //* creation
    TimeStamp creation_;

    //* modification
    TimeStamp modification_;

    //* last call
    TimeStamp lastCalled_;

    //@}

};

Q_DECLARE_OPERATORS_FOR_FLAGS( TaskAttributes::Flags );

#endif
