#ifndef TaskBarAttributes_h
#define TaskBarAttributes_h

// $Id: TaskBarAttributes.h,v 1.43 2014/09/12 08:43:12 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Options.h"
#include "TaskAttributes.h"

#include <QPoint>
#include <QDomDocument>
#include <QDomElement>
#include <QMap>
#include <QList>

//! single TaskAttributes attributesrmation
class TaskBarAttributes: public Counter
{

    public:

    //! task attributes list
    class List: public QList<TaskBarAttributes>, public Counter
    {
        public:

        //! constructor
        List( void ):
            Counter( "TaskBarAttributes::List" )
        {}

        //! copy constructor
        List( const QList<TaskBarAttributes>& other ):
            QList<TaskBarAttributes>( other ),
            Counter( "TaskBarAttributes::List" )
        {}

        //! destructor
        virtual ~List( void )
        {}

        //! returns true if at least one item in lists ::differs from the other
        bool differs( const QList<TaskBarAttributes>& ) const;

    };

    //! constructor
    TaskBarAttributes( const QDomElement& = QDomElement() );

    //! dom element
    QDomElement domElement( QDomDocument& ) const;

    //! id
    unsigned int id() const
    { return id_; }

    //! equal to operator
    /*!
    only the options are compared. The TaskAttributesList
    must be compared independantly
    */
    bool operator == (const TaskBarAttributes& other ) const
    {
        return
            id() == other.id() &&
            name() == other.name() &&
            layout() == other.layout() &&
            mode() == other.mode() &&
            numColumns() == other.numColumns() &&
            zoomDelay() == other.zoomDelay() &&
            positionMode() == other.positionMode() &&
            scale() == other.scale() &&
            flags() == other.flags();
    }

    //! true if attributes differ from other.
    /*! test name, flags, and attributes list. Does not test Id */
    bool differs( const TaskBarAttributes& ) const;

    //! return true if attributes are different up to the point when tasks are to be updated
    bool needTaskUpdate( const TaskBarAttributes& );

    //! name
    const QString& name( void ) const
    { return name_; }

    //! name
    void setName( const QString& value )
    { name_ = value; }

    //!@name layout
    //@{

    //! layout enum
    enum Layout
    {
        TopLeft = 1<<0,
        Center = 1<<1,
        BottomRight = 1<<2,
        Vertical = 1<<3,
        Horizontal = 1<<4,
        Box = 1<<5,

        VerticalCenter = Vertical | Center,
        VerticalLeft = Vertical | TopLeft,
        VerticalRight = Vertical | BottomRight,

        HorizontalCenter = Horizontal | Center,
        HorizontalTop = Horizontal | TopLeft,
        HorizontalBottom = Horizontal | BottomRight

    };

    Q_DECLARE_FLAGS( LayoutFlags, Layout )

    //! Layout
    Layout layout() const
    { return layout_; }

    //! map layout to layout name
    using LayoutNameMap = QMap< Layout, QString >;

    //! Layout
    QString layoutName() const
    { return layoutName( layout() ); }

    //! layout names
    static QString layoutName( const Layout& layout )
    {
        if( _layoutNames().empty() ) _initializeLayoutNames();
        LayoutNameMap::const_iterator iter( _layoutNames().find( layout ) );
        Q_ASSERT( iter != _layoutNames().end() );
        return iter.value();
    }

    //! layout names
    static const LayoutNameMap& layoutNames()
    {
        if( _layoutNames().empty() ) _initializeLayoutNames();
        return _layoutNames();
    }

    //! Layout
    void setLayout( const Layout& layout )
    { layout_ = layout; }

    //! Layout
    void setLayout( const QString& layout );

    //@}

    //!@name mode
    //@{

    //! zoom mode
    enum Mode
    {
        Fixed,
        Minimum,
        Parabolic

    };

    //! Mode
    const Mode& mode() const
    { return mode_; }

    //! Mode
    QString modeName() const
    { return modeName( mode() ); }

    //! mode names
    static QString modeName( const Mode& mode )
    {
        ModeNameMap::const_iterator iter( modeNames().find( mode ) );
        Q_ASSERT( iter != modeNames().end() );
        return iter.value();
    }

    //! map mode to mode name
    using ModeNameMap = QMap< Mode, QString >;

    //! layout names
    static const ModeNameMap& modeNames()
    {

        if( _modeNames().empty() ) _initializeModeNames();
        return _modeNames();
    }

    //! Mode
    void setMode( const Mode& mode )
    { mode_ = mode; }

    //! Mode
    void setMode( const QString& mode );

    //@}

    //! number of columns
    const unsigned int& numColumns( void ) const
    { return nColumns_; }

    //! number of columns
    void setNumColumns( const unsigned int value )
    { nColumns_ = value; }

    //!@name positions
    //@{

    //! predefined positions
    enum { n_positions_ = 8 };
    enum Position
    {
        PositionAny = 0,
        PositionTop = 1<<0,
        PositionBottom = 1<<1,
        PositionLeft = 1<<2,
        PositionRight = 1<<3,
        PositionCenter = 1<<4,
        PositionTopLeft      = PositionTop | PositionLeft,
        PositionTopCenter    = PositionTop | PositionCenter,
        PositionTopRight     = PositionTop | PositionRight,
        PositionBottomLeft   = PositionBottom | PositionLeft,
        PositionBottomCenter = PositionBottom | PositionCenter,
        PositionBottomRight  = PositionBottom | PositionRight,
        PositionLeftCenter   = PositionCenter | PositionLeft,
        PositionRightCenter  = PositionCenter | PositionRight
    };

    Q_DECLARE_FLAGS( PositionMode, Position );

    //! map mode to mode name
    using PositionNameMap = QMap< PositionMode, QString >;

    //! layout names
    static const PositionNameMap& positionNames()
    {
        if( _positionNames().empty() ) _initializePositionNames();
        return _positionNames();
    }

    //! Layout
    QString positionName() const
    { return positionName( positionMode() ); }

    //! layout names
    static QString positionName( PositionMode );

    //! position mode
    PositionMode positionMode( void ) const
    { return positionMode_; }

    //! podiyion mode
    void setPositionMode( const QString& );

    //! podiyion mode
    void setPositionMode( PositionMode position )
    { positionMode_ = position; }

    //@}

    //! position
    QPoint position( void ) const
    { return position_; }

    //! position
    void setPosition( const QPoint& point )
    { position_ = point;  }

    //! position
    void setX( const int &x )
    { position_.setX(x); }

    //! position
    void setY( const int &y )
    { position_.setY(y); }

    //! zoom delay
    const unsigned int& zoomDelay( void ) const
    { return zoomDelay_; }

    //! zoom delay
    void setZoomDelay( const unsigned int& delay )
    { zoomDelay_ = delay; }

    //!@name flags
    //@{

    //! flags
    enum Flag
    {

        None = 0,
        ShowName = 1<<0,
        LockPosition = 1<<1,
        LockButtons = 1<<2,
        ExpandSeparators = 1<<3,
        StaysOnTop = 1<<4,
        AutoHide = 1<<5,
        RoundCorners = 1<<6,
        Sticky = 1<<8,
        HideFromTaskBar = 1<<9,
        ZoomEnabled = 1<<10
    };

    Q_DECLARE_FLAGS( Flags, Flag );

    //! flags
    void setFlag( const Flag& flag, const bool& value = true )
    {
        if( value ) flags_ |= flag;
        else flags_ &= (~flag);
    }

    //! flags
    void setFlags( Flags flags )
    { flags_ = flags; }

    //! flags
    bool hasFlag( Flags flags ) const
    { return flags_ &flags; }

    //! flags
    Flags flags( void ) const
    { return flags_; }

    //@}

    //! scale
    const double& scale( void ) const
    { return scale_; }

    //! scale
    void setScale( const double& value )
    { scale_ = value; }

    //! task attributes
    const TaskAttributes::List& taskAttributesList( void ) const
    { return taskList_; }

    //! task attributes
    void setTaskAttributesList( const TaskAttributes::List& taskList )
    {
        taskList_ = taskList;
        needTaskUpdate_ = true;
    }

    //! clear task attributes
    void clear( void )
    { taskList_.clear(); }

    //! task attributes
    void addTaskAttributes( const TaskAttributes& attributes )
    { taskList_.append( attributes ); }

    private:

    //! mode names
    static void _initializeModeNames( void );

    //! layout names
    static void _initializeLayoutNames( void );

    //! position names
    static void _initializePositionNames( void );

    //! id count
    static unsigned int& _idCount( void );

    //! map mode to name
    static ModeNameMap& _modeNames( void );

    //! map layout to name
    static LayoutNameMap& _layoutNames( void );

    //! map position mode to name
    static PositionNameMap& _positionNames( void );

    //! id
    unsigned int id_;

    //! name
    QString name_;

    //! layout
    Layout layout_;

    //! mode
    Mode mode_;

    //! number of columns
    unsigned int nColumns_;

    //!@name position
    //@{

    //! predefined position
    PositionMode positionMode_;

    //! position
    QPoint position_;

    //@}

    //! delay
    unsigned zoomDelay_;

    //! flags (bitwise or of Flag object)
    Flags flags_;

    //! zoom scale (Parabolic mode)
    double scale_;

    //! task attributes
    TaskAttributes::List taskList_;

    //! true when tasks needs to be updated
    bool needTaskUpdate_;

    //! dump to Xml
    friend QTextStream& operator << (QTextStream& out, const TaskBarAttributes& );

};

Q_DECLARE_OPERATORS_FOR_FLAGS( TaskBarAttributes::LayoutFlags );
Q_DECLARE_OPERATORS_FOR_FLAGS( TaskBarAttributes::PositionMode );
Q_DECLARE_OPERATORS_FOR_FLAGS( TaskBarAttributes::Flags );

#endif
