// $Id: ToolTipWidget.cpp,v 1.11 2015/01/08 23:20:47 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ToolTipWidget.h"
#include "ToolTipWidget.moc"
#include "CustomPixmap.h"
#include "Debug.h"
#include "IconSize.h"
#include "QtUtil.h"
#include "Singleton.h"
#include "XmlOptions.h"

#include <QLayout>
#include <QPainter>
#include <QStyleOptionFrame>
#include <QToolTip>

//_______________________________________________________________________
ToolTipWidget::ToolTipWidget( QWidget* parent ):
    BaseToolTipWidget( parent ),
    pixmapSize_( IconSize( IconSize::Maximum ) )
{

    // layout
    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin( 15 );
    hLayout->setSpacing( 10 );
    setLayout( hLayout );

    hLayout->addWidget( iconLabel_ = new QLabel( this ) );
    iconLabel_->setAlignment( Qt::AlignHCenter|Qt::AlignTop );

    QVBoxLayout* vLayout = new QVBoxLayout();
    vLayout->setMargin( 0 );
    vLayout->setSpacing( 5 );
    vLayout->addStretch( 1 );
    hLayout->addLayout( vLayout );

    // title
    vLayout->addWidget( titleLabel_ = new QLabel( this ) );
    titleLabel_->setAlignment( Qt::AlignCenter );
    titleLabel_->setFont( QtUtil::titleFont( titleLabel_->font() ) );
    titleLabel_->setMargin( 1 );

    // frame
    separator_ = new QFrame( this );
    separator_->setFrameStyle( QFrame::HLine );
    vLayout->addWidget( separator_ );

    // comments
    vLayout->addWidget( commentsLabel_ = new QLabel( this ) );
    commentsLabel_->setAlignment( Qt::AlignCenter );
    commentsLabel_->setMargin( 1 );
    vLayout->addStretch( 1 );

    separator_->hide();
    commentsLabel_->hide();

    connect( Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();

}

//__________________________________________________
void ToolTipWidget::setTaskAttributes( const TaskAttributes& attributes )
{

    // try load pixmap from cache
    Cache::const_iterator iter( cache_.find( attributes.iconName() ) );
    if( iter != cache_.end() ) iconLabel_->setPixmap( iter.value() );
    else {

        // if not found, create a new pixmap,
        CustomPixmap pixmap( attributes.iconName() );
        if( pixmap.isNull() ) pixmap = CustomPixmap( pixmapSize_, CustomPixmap::Transparent );
        else pixmap = pixmap.scaled( pixmapSize_, Qt::KeepAspectRatio, Qt::SmoothTransformation );

        // insert to pixmap
        cache_.insert( attributes.iconName(), pixmap );

        // assign to local
        iconLabel_->setPixmap( pixmap );

    }

    // title and comment
    titleLabel_->setText( attributes.name() );

    if( !attributes.comments().isEmpty() )
    {

        separator_->show();
        commentsLabel_->show();
        commentsLabel_->setText( attributes.comments() );

    } else {

        separator_->hide();
        commentsLabel_->hide();

    }

    _adjustPosition();

}

//____________________________________
void ToolTipWidget::_updateConfiguration( void )
{
    Debug::Throw( "ToolTipWidget::_updateConfiguration.\n" );

    // pixmap size
    int size = (int)( XmlOptions::get().get<int>( "TOOLTIP_HEIGHT" ) );
    pixmapSize_ = QSize( size, size );

    return;
}
